"""vcard package"""
__package__ = "vcard"
__author__ = "Victor Engmark"
__copyright__ = "Copyright (C) 2010-2019 Victor Engmark"
__credits__ = ["Victor Engmark", "Zearin"]
__maintainer__ = "Victor Engmark"
__email__ = "victor.engmark@gmail.com"
__license__ = "AGPLv3+"
__url__ = "https://gitlab.com/victor-engmark/vcard"
__version__ = "0.12.0"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
vCard v3.0 (RFC 2426) validating functions

Should contain all the general purpose validation code extracted from
standards.
"""

import re
import warnings
from urllib.parse import urlparse

from dateutil.parser import isoparse

from .vcard_definitions import (
    DOUBLE_QUOTE_CHARACTER,
    ESCAPED_CHARACTERS,
    ID_CHARACTERS,
    QUOTE_SAFE_CHARACTERS,
    SAFE_CHARACTERS,
    SPACE_CHARACTER,
)

from .vcard_errors import (  # Error literals; Functions; Classes
    NOTE_INVALID_DATE,
    NOTE_INVALID_LANGUAGE_VALUE,
    NOTE_INVALID_PARAMETER_NAME,
    NOTE_INVALID_PARAMETER_VALUE,
    NOTE_INVALID_SUB_VALUE,
    NOTE_INVALID_SUB_VALUE_COUNT,
    NOTE_INVALID_TEXT_VALUE,
    NOTE_INVALID_TIME_ZONE,
    NOTE_INVALID_URI,
    NOTE_INVALID_VALUE,
    NOTE_INVALID_VALUE_COUNT,
    NOTE_INVALID_X_NAME,
    NOTE_MISMATCH_PARAMETER,
    NOTE_MISSING_PARAMETER,
    NOTE_NON_EMPTY_PARAMETER,
    WARN_DEFAULT_TYPE_VALUE,
    WARN_INVALID_EMAIL_TYPE,
    WARN_MULTIPLE_NAMES,
    VCardError,
    VCardItemCountError,
    VCardNameError,
    VCardValueError,
    show_warning,
)

VALID_DATE = re.compile(r"^\d{4}-?\d{2}-?\d{2}$")
VALID_TIMEZONE = re.compile(r"^(Z|[+-]\d{2}:?\d{2})$")
VALID_TIME_WITH_TIMEZONE = re.compile(r"^(\d{2}:?\d{2}:?\d{2}(?:,\d+)?)(.*)$")
VALID_LANGUAGE_TAG = re.compile(r"^([a-z]{1,8})(-[a-z]{1,8})*$")
VALID_X_NAME = re.compile(r"^X-[{0}]+$".format(re.escape(ID_CHARACTERS)))
VALID_PRESENTATION_TEXT = re.compile("^[{0}]*$".format(re.escape(SAFE_CHARACTERS)))
VALID_TEXT = re.compile(
    "^([{0}:{1}]|(\\\\[{2}]))*$".format(
        re.escape(SAFE_CHARACTERS), DOUBLE_QUOTE_CHARACTER, re.escape(ESCAPED_CHARACTERS)
    )
)
VALID_QUOTED_STRING = re.compile("^{0}[{1}]{0}$".format(DOUBLE_QUOTE_CHARACTER, re.escape(QUOTE_SAFE_CHARACTERS)))
VALID_FLOAT = re.compile(r"^[+-]?\d+(\.\d+)?$")

LABEL_TYPE_VALUES = ("dom", "intl", "postal", "parcel", "home", "work", "pref")
TELEPHONE_TYPE_VALUES = (
    "home",
    "msg",
    "work",
    "pref",
    "voice",
    "fax",
    "cell",
    "video",
    "pager",
    "bbs",
    "modem",
    "car",
    "isdn",
    "pcs",
)
EMAIL_TYPE_VALUES = ("internet", "x400", "pref", "dom", "intl", "postal", "parcel", "home", "work")


def _expect_no_parameters(property_):
    parameters = __get_parameters(property_)
    if parameters is not None:
        raise VCardItemCountError("{0}: {1}".format(NOTE_NON_EMPTY_PARAMETER, parameters), {})


def _expect_parameters(property_):
    if __get_parameters(property_) is None:
        raise VCardItemCountError(NOTE_MISSING_PARAMETER, {})


def _expect_value_count(values, count):
    if len(values) != count:
        raise VCardItemCountError("{0}: {1:d} (expected {2})".format(NOTE_INVALID_VALUE_COUNT, len(values), count), {})


def _expect_sub_value_count(sub_values, count):
    if len(sub_values) != count:
        raise VCardItemCountError(
            "{0}: {1:d} (expected {2})".format(NOTE_INVALID_SUB_VALUE_COUNT, len(sub_values), count), {}
        )


def validate_date(text):
    """
    Based on http://tools.ietf.org/html/rfc2425#section-5.8.4 and the fact
    that it specifies a subset of ISO 8601.

    @param text: String
    """
    if VALID_DATE.match(text) is None:
        raise VCardValueError(NOTE_INVALID_DATE, {"String": text})

    try:
        isoparse(text)
    except ValueError:
        raise VCardValueError(NOTE_INVALID_DATE, {"String": text})


def validate_time_zone(text):
    """
    Based on http://tools.ietf.org/html/rfc2425#section-5.8.4 and the fact
    that it specifies a subset of ISO 8601.

    @param text: String

    Examples:
    >>> validate_time_zone('Z')
    >>> validate_time_zone('+01:00')
    >>> validate_time_zone('-12:30')
    >>> validate_time_zone('+23:59')
    >>> validate_time_zone('-0001')
    >>> validate_time_zone('-00:30')
    >>> validate_time_zone('+00:30')
    >>> validate_time_zone('Z+01:00') # Can't combine Z and offset
    ... # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid time zone ...
    String: Z+01:00
    >>> validate_time_zone('+1:00') # Need preceding zero
    ... # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid time zone ...
    String: +1:00
    >>> validate_time_zone('0100') # Need + or -
    ... # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid time zone ...
    String: 0100
    >>> validate_time_zone('01') # Need colon and minutes
    ... # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid time zone ...
    String: 01
    >>> validate_time_zone('01:') # Need minutes
    ... # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid time zone ...
    String: 01:
    >>> validate_time_zone('01:1') # Need preceding zero
    ... # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid time zone ...
    """
    if not VALID_TIMEZONE.match(text):
        raise VCardValueError(NOTE_INVALID_TIME_ZONE, {"String": text})

    try:
        isoparse(f"2000-01-01 00:00:00.0{text}")
    except ValueError:
        raise VCardValueError(NOTE_INVALID_TIME_ZONE, {"String": text})


def validate_language_tag(text):
    """
    langval, as defined by RFC 1766 <http://tools.ietf.org/html/rfc1766>

    @param text: String

    Examples:
    >>> validate_language_tag('en')
    >>> validate_language_tag('-US') # Need primary tag # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid language (See RFC 1766 section 2 for details)
    String: -us
    >>> validate_language_tag('en-') # Can't end with dash # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid language (See RFC 1766 section 2 for details)
    String: en-
    >>> validate_language_tag('en-US')

    """
    text = text.lower()  # Case insensitive

    if VALID_LANGUAGE_TAG.match(text) is None:
        raise VCardValueError(NOTE_INVALID_LANGUAGE_VALUE, {"String": text})

    # TODO: Extend to validate according to referenced ISO/RFC standards


def validate_x_name(text):
    """
    @param text: Single parameter name

    Examples:
    >>> validate_x_name('X-abc')
    >>> validate_x_name('X-' + ID_CHARACTERS)
    >>> validate_x_name('X-') # Have to have more characters # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardNameError: Invalid X-name (See RFC 2426 section 4 for x-name syntax)
    String: X-
    >>> validate_x_name('') # Have to start with X- #doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    ...
    VCardNameError: Invalid X-name (See RFC 2426 section 4 for x-name syntax)
    ...
    >>> validate_x_name('x-abc') # X must be upper case # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardNameError: Invalid X-name (See RFC 2426 section 4 for x-name syntax)
    String: x-abc
    >>> validate_x_name('foo') # Have to start with X- # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardNameError: Invalid X-name (See RFC 2426 section 4 for x-name syntax)
    String: foo
    """
    if VALID_X_NAME.match(text) is None:
        raise VCardNameError(NOTE_INVALID_X_NAME, {"String": text})


def validate_presentation_text(text):
    """
    ptext, as described on page 28
    <http://tools.ietf.org/html/rfc2426#section-4>

    @param text: String

    Examples:
    >>> validate_presentation_text('')
    >>> validate_presentation_text(SAFE_CHARACTERS)
    >>> validate_presentation_text(u'\u000B') #doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid parameter value ...
    String: ...
    """
    if VALID_PRESENTATION_TEXT.match(text) is None:
        raise VCardValueError(NOTE_INVALID_PARAMETER_VALUE, {"String": text})


def validate_text_value(text):
    """
    text-value, as described on page 37
    <http://tools.ietf.org/html/rfc2426#section-4>

    @param text: String

    Examples:
    >>> validate_text_value('')
    >>> validate_text_value('\\\\,')
    >>> validate_text_value(SAFE_CHARACTERS)
    >>> validate_text_value('\\\\n')
    >>> validate_text_value(';') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid text value (See RFC 2426 section 4 for details)
    String: ...
    >>> validate_text_value('\\\\\\\\;') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid text value (See RFC 2426 section 4 for details)
    String: ...
    """
    if VALID_TEXT.match(text) is None:
        raise VCardValueError(NOTE_INVALID_TEXT_VALUE, {"String": text})


def validate_quoted_string(text):
    """
    quoted-string, as described on page 28
    <http://tools.ietf.org/html/rfc2426#section-4>

    @param text: String

    Examples:
    >>> validate_quoted_string(DOUBLE_QUOTE_CHARACTER + QUOTE_SAFE_CHARACTERS[0] + DOUBLE_QUOTE_CHARACTER)
    >>> validate_quoted_string(
    ... DOUBLE_QUOTE_CHARACTER + DOUBLE_QUOTE_CHARACTER) # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid parameter value ...
    >>> validate_quoted_string(
    ... DOUBLE_QUOTE_CHARACTER + QUOTE_SAFE_CHARACTERS[-1]*2 + DOUBLE_QUOTE_CHARACTER
    ... ) # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid parameter value ...
    String: "ÿÿ"
    """
    if VALID_QUOTED_STRING.match(text) is None:
        raise VCardValueError(NOTE_INVALID_PARAMETER_VALUE, {"String": text})


def validate_param_value(text):
    """
    param-value, as described on page 28
    <http://tools.ietf.org/html/rfc2426#section-4>

    @param text: Single parameter value

    Examples:
    >>> validate_param_value('')
    >>> validate_param_value(SAFE_CHARACTERS)
    >>> validate_param_value(DOUBLE_QUOTE_CHARACTER + QUOTE_SAFE_CHARACTERS[0] + DOUBLE_QUOTE_CHARACTER)
    >>> validate_param_value(
    ... DOUBLE_QUOTE_CHARACTER + DOUBLE_QUOTE_CHARACTER) # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid parameter value ...
    String: ""
    """
    try:
        validate_presentation_text(text)
    except VCardValueError:
        try:
            validate_quoted_string(text)
        except VCardValueError:
            raise VCardValueError(NOTE_INVALID_PARAMETER_VALUE, {"String": text})


def validate_text_parameter(parameter):
    """
    text-param, as described on page 35
    <http://tools.ietf.org/html/rfc2426#section-4>

    @param parameter: Single parameter, as returned by get_vcard_property_param

    Examples:
    >>> validate_text_parameter(['VALUE', {'ptext'}])
    """
    param_name = parameter[0].upper()
    param_values = parameter[1]

    if param_name == "VALUE":
        if param_values != {"ptext"}:
            raise VCardValueError("{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_values), {})
        return
    elif param_name == "LANGUAGE":
        if len(param_values) != 1:
            raise VCardValueError("{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_values), {})
        for param_value in param_values:
            validate_language_tag(param_value)
    else:
        validate_x_name(param_name)
        if len(param_values) != 1:
            raise VCardValueError("{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_values), {})
        validate_param_value(param_values[0])


def validate_float(text):
    """
    float value, as described on page 10 of RFC 2425
    <http://tools.ietf.org/html/rfc2425#section-5.8.4>

    Examples:
    >>> validate_float('12')
    >>> validate_float('12.345')
    >>> validate_float('+12.345')
    >>> validate_float('-12.345')
    >>> validate_float('12.') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
        ...
    VCardValueError: Invalid sub-value ...
    >>> validate_float('.12') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
        ...
    VCardValueError: Invalid sub-value ...
    >>> validate_float('foo') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
        ...
    VCardValueError: Invalid sub-value ...
    >>> validate_float('++12.345') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
        ...
    VCardValueError: Invalid sub-value ...
    >>> validate_float('--12.345') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
        ...
    VCardValueError: Invalid sub-value ...
    >>> validate_float('12.34.5') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
        ...
    VCardValueError: Invalid sub-value ...
    """
    if VALID_FLOAT.match(text) is None:
        raise VCardValueError("{0}, expected float value: {1}".format(NOTE_INVALID_SUB_VALUE, text), {})


def validate_uri(text):
    """
    genericurl, as described in RFC 1738
    <http://tools.ietf.org/html/rfc1738#section-5>.
    @param text: Single parameter value

    Examples:
    >>> validate_uri('http://example.org/')
    >>> validate_uri('http\\://example.org/') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid URI ...
    String: http\\://example.org/
    >>> validate_uri('http:') # doctest: +ELLIPSIS +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    VCardValueError: Invalid URI ...
    String: http:
    """
    parts = urlparse(text)
    if parts[0] == "" or (parts[1] == "" and parts[2] == ""):
        raise VCardValueError(NOTE_INVALID_URI, {"String": text})


def validate_vcard_property(property_):
    """
    Checks any property according to
    <http://tools.ietf.org/html/rfc2426#section-3> and
    <http://tools.ietf.org/html/rfc2426#section-4>. Checks are grouped by
    property to allow easy overview rather than a short function.

    @param property_: Formatted property
    """
    property_name = property_.name.upper()

    try:
        if property_name in ("BEGIN", "END"):
            # <http://tools.ietf.org/html/rfc2426#section-2.1.1>
            _expect_no_parameters(property_)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            if property_.values[0][0].lower() != "vcard":
                raise VCardValueError(
                    '{0}: {1} (expected "VCARD")'.format(NOTE_INVALID_VALUE, property_.values[0][0]), {}
                )

        if property_name == "NAME":
            # <http://tools.ietf.org/html/rfc2426#section-2.1.2>
            _expect_no_parameters(property_)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            validate_text_value(property_.values[0][0])

        if property_name == "PROFILE":
            # <http://tools.ietf.org/html/rfc2426#section-2.1.3>
            _expect_no_parameters(property_)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            if property_.values[0][0].lower() != "vcard":
                raise VCardValueError(
                    '{0}: {1} (expected "VCARD")'.format(NOTE_INVALID_VALUE, property_.values[0][0]), {}
                )
            validate_text_value(property_.values[0][0])

        if property_name == "SOURCE":
            # <http://tools.ietf.org/html/rfc2426#section-2.1.4>
            _expect_parameters(property_)
            for parameter_name, param_values in property_.parameters.items():
                if parameter_name.upper() == "VALUE":
                    if param_values != {"uri"}:
                        raise VCardValueError("{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_values), {})
                    if "CONTEXT" in property_.parameters:
                        raise VCardValueError(
                            "{0}: {1} and {2}".format(NOTE_MISMATCH_PARAMETER, "VALUE", "CONTEXT"), {}
                        )
                elif parameter_name.upper() == "CONTEXT":
                    if param_values != {"word"}:
                        raise VCardValueError("{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_values), {})
                    if "VALUE" in property_.parameters:
                        raise VCardValueError(
                            "{0}: {1} and {2}".format(NOTE_MISMATCH_PARAMETER, "VALUE", "CONTEXT"), {}
                        )
                else:
                    raise VCardNameError("{0}: {1}".format(NOTE_INVALID_PARAMETER_NAME, parameter_name), {})

        if property_name == "FN":
            # <http://tools.ietf.org/html/rfc2426#section-3.1.1>
            if property_.parameters is not None:
                for parameter in property_.parameters.items():
                    validate_text_parameter(parameter)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            validate_text_value(property_.values[0][0])

        elif property_name == "VERSION":
            _expect_no_parameters(property_)
            _expect_value_count(property_.values, 1)
            if property_.values[0][0] != "3.0":
                raise VCardValueError(
                    '{0}: {1} (expected "3.0")'.format(NOTE_INVALID_VALUE, property_.values[0][0]), {}
                )
        elif property_name == "N":
            # <http://tools.ietf.org/html/rfc2426#section-3.1.2>
            if property_.parameters is not None:
                for parameter in property_.parameters.items():
                    validate_text_parameter(parameter)
            _expect_value_count(property_.values, 5)
            # Should names be split?
            for names in property_.values:
                warnings.showwarning = show_warning
                for name in names:
                    validate_text_value(name)
                    if (
                        name.find(SPACE_CHARACTER) != -1
                        and "".join(["".join(names) for names in property_.values]) != name
                    ):
                        # Space in name
                        # Not just a single name
                        warnings.warn("{0}: {1}".format(WARN_MULTIPLE_NAMES, name.encode("utf-8")))

        elif property_name == "NICKNAME":
            # <http://tools.ietf.org/html/rfc2426#section-3.1.3>
            if property_.parameters is not None:
                for parameter in property_.parameters.items():
                    validate_text_parameter(parameter)
            _expect_value_count(property_.values, 1)

        elif property_name in ["PHOTO", "LOGO"]:
            # <http://tools.ietf.org/html/rfc2426#section-3.1.4>
            # <http://tools.ietf.org/html/rfc2426#section-3.5.4>
            _expect_parameters(property_)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            for parameter_name, param_values in property_.parameters.items():
                if parameter_name.upper() == "ENCODING":
                    if param_values != {"b"}:
                        raise VCardValueError("{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_values), {})
                    if "VALUE" in property_.parameters:
                        raise VCardValueError(
                            "{0}: {1} and {2}".format(NOTE_MISMATCH_PARAMETER, "ENCODING", "VALUE"), {}
                        )
                elif parameter_name.upper() == "TYPE" and "ENCODING" not in property_.parameters:
                    raise VCardItemCountError("{0}: {1}".format(NOTE_MISSING_PARAMETER, "ENCODING"), {})
                elif parameter_name.upper() == "VALUE":
                    if param_values != {"uri"}:
                        raise VCardValueError("{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_values), {})
                    else:
                        validate_uri(property_.values[0][0])
                elif parameter_name.upper() not in ["ENCODING", "TYPE", "VALUE"]:
                    raise VCardNameError("{0}: {1}".format(NOTE_INVALID_PARAMETER_NAME, parameter_name), {})

        elif property_name == "BDAY":
            # <http://tools.ietf.org/html/rfc2426#section-3.1.5>
            _expect_no_parameters(property_)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            validate_date(property_.values[0][0])

        elif property_name == "ADR":
            # <http://tools.ietf.org/html/rfc2426#section-3.2.1>
            _expect_value_count(property_.values, 7)
            if property_.parameters is not None:
                for parameter_name, param_values in property_.parameters.items():
                    if parameter_name.upper() == "TYPE":
                        for param_sub_value in param_values:
                            if param_sub_value not in LABEL_TYPE_VALUES:
                                raise VCardValueError(
                                    "{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_sub_value), {}
                                )
                        if param_values == {"intl", "postal", "parcel", "work"}:
                            warnings.warn("{0}: {1}".format(WARN_DEFAULT_TYPE_VALUE, property_.values))
                    else:
                        validate_text_parameter(property_)

        elif property_name == "LABEL":
            # <http://tools.ietf.org/html/rfc2426#section-3.2.2>
            if property_.parameters is not None:
                for parameter_name, param_values in property_.parameters.items():
                    if parameter_name.upper() == "TYPE":
                        for param_sub_value in param_values:
                            if param_sub_value not in LABEL_TYPE_VALUES:
                                raise VCardValueError(
                                    "{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_sub_value), {}
                                )
                        if param_values == {"intl", "postal", "parcel", "work"}:
                            warnings.warn("{0}: {1}".format(WARN_DEFAULT_TYPE_VALUE, property_.values))
                    else:
                        validate_text_parameter(property_)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            validate_text_value(property_.values[0][0])

        elif property_name == "TEL":
            # <http://tools.ietf.org/html/rfc2426#section-3.3.1>
            if property_.parameters is not None:
                for parameter_name, param_values in property_.parameters.items():
                    if parameter_name.upper() == "TYPE":
                        for param_sub_value in param_values:
                            if param_sub_value.lower() not in TELEPHONE_TYPE_VALUES:
                                raise VCardValueError(
                                    "{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_sub_value), {}
                                )
                        if set([value.lower() for value in param_values]) == {"voice"}:
                            warnings.warn("{0}: {1}".format(WARN_DEFAULT_TYPE_VALUE, property_.values))
                    else:
                        raise VCardNameError("{0}: {1}".format(NOTE_INVALID_PARAMETER_NAME, parameter_name), {})
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)

        elif property_name == "EMAIL":
            # <http://tools.ietf.org/html/rfc2426#section-3.3.2>
            if property_.parameters is not None:
                for parameter_name, param_values in property_.parameters.items():
                    if parameter_name.upper() == "TYPE":
                        for param_sub_value in param_values:
                            if param_sub_value.lower() not in EMAIL_TYPE_VALUES:
                                warnings.warn("{0}: {1}".format(WARN_INVALID_EMAIL_TYPE, param_sub_value))
                        if set([value.lower() for value in param_values]) == {"internet"}:
                            warnings.warn("{0}: {1[values]}".format(WARN_DEFAULT_TYPE_VALUE, property_))
                    else:
                        raise VCardNameError("{0}: {1}".format(NOTE_INVALID_PARAMETER_NAME, parameter_name), {})
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            validate_text_value(property_.values[0][0])

        elif property_name == "MAILER":
            # <http://tools.ietf.org/html/rfc2426#section-3.3.3>
            _expect_no_parameters(property_)
            _expect_value_count(property_.values, 1)
            _expect_value_count(property_.values[0], 1)
            validate_text_value(property_.values[0][0])

        elif property_name == "TZ":
            # <http://tools.ietf.org/html/rfc2426#section-3.4.1>
            _expect_no_parameters(property_)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            value = property_.values[0][0]
            validate_time_zone(value)

        elif property_name == "GEO":
            # <http://tools.ietf.org/html/rfc2426#section-3.4.2>
            _expect_no_parameters(property_)
            _expect_value_count(property_.values, 2)
            # can the following be...
            #   _expect_sub_value_count(property_.values[0], 2)
            # ...?
            for value in property_.values:
                if len(value) != 1:
                    raise VCardItemCountError(
                        "{0}: {1:d} (expected 1)".format(NOTE_INVALID_SUB_VALUE_COUNT, len(property_.values[0])), {}
                    )
                validate_float(value[0])

        elif property_name == "TITLE":
            # <http://tools.ietf.org/html/rfc2426#section-3.5.1>
            if property_.parameters is not None:
                for parameter in property_.parameters.items():
                    validate_text_parameter(parameter)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            validate_text_value(property_.values[0][0])

        elif property_name == "ROLE":
            # <http://tools.ietf.org/html/rfc2426#section-3.5.2>
            if property_.parameters is not None:
                for parameter in property_.parameters.items():
                    validate_text_parameter(parameter)
            _expect_value_count(property_.values, 1)
            _expect_sub_value_count(property_.values[0], 1)
            validate_text_value(property_.values[0][0])

        elif property_name == "AGENT":
            # <http://tools.ietf.org/html/rfc2426#section-3.5.4>
            if property_.parameters is not None:
                for parameter_name, param_values in property_.parameters.items():
                    if parameter_name.upper() != "VALUE":
                        raise VCardNameError("{0}: {1}".format(NOTE_INVALID_PARAMETER_NAME, param_values), {})
                    if param_values != {"uri"}:
                        raise VCardValueError("{0}: {1}".format(NOTE_INVALID_PARAMETER_VALUE, param_values), {})
                _expect_value_count(property_.values, 1)
                # can this be...
                #   _expect_sub_value_count(property_.values[0], 1)
                # ...?
                for value in property_.values:
                    if len(value) != 1:
                        raise VCardItemCountError(
                            "{0}: {1:d} (expected 1)".format(NOTE_INVALID_SUB_VALUE_COUNT, len(property_.values[0])), {}
                        )
                    validate_uri(value[0])
            else:
                # Inline vCard object
                pass  # TODO: Un-escape and validate value

        elif property_name == "URL":
            # <http://tools.ietf.org/html/rfc2426#section-3.6.8>
            _expect_no_parameters(property_)
            _expect_value_count(property_.values, 1)
            validate_uri(property_.values[0][0])

    except VCardError as error:
        error.context["Property"] = property_name
        err_type = type(error)
        raise err_type(error.message, error.context)


def __get_parameters(property_):
    return property_.parameters



# -*- coding: utf-8 -*-

import sys

# Error literals

# General
NOTE_EMPTY_VCARD = "vCard is empty"

# Lines
NOTE_CONTINUATION_AT_START = "Continuation line at start of vCard (See RFC 2425 section 5.8.1 for line folding details)"
NOTE_INVALID_LINE_SEPARATOR = "Invalid line ending; should be \\r\\n (See RFC 2426 section 2.4.2 for details)"
NOTE_DOT_AT_LINE_START = "Dot at start of line without group name (See RFC 2426 section 4 for group syntax)"
NOTE_MISSING_GROUP = "Missing group (See RFC 2426 section 4 for contentline syntax)"

# Item counts & Length
NOTE_NON_EMPTY_PARAMETER = "Property should not have parameters (See RFC 2426 section 3 for details)"
NOTE_INVALID_SUB_VALUE_COUNT = "Invalid sub-value count (See RFC 2426 section 3 for details)"
NOTE_INVALID_VALUE_COUNT = "Invalid value count (See RFC 2426 section 3 for details)"
NOTE_MISSING_PARAMETER = "Parameter missing (See RFC 2426 section 3 for details)"
NOTE_MISSING_PARAM_VALUE = "Parameter value missing (See RFC 2426 section 3 for details)"
NOTE_MISSING_PROPERTY = "Mandatory property missing (See RFC 2426 section 5 for details)"
NOTE_MISSING_VALUE_STRING = "Missing value string (See RFC 2426 section 4 for contentline syntax)"

# Names
NOTE_INVALID_PROPERTY_NAME = "Invalid property name (See RFC 2426 section 4 for name syntax)"
NOTE_INVALID_X_NAME = "Invalid X-name (See RFC 2426 section 4 for x-name syntax)"
NOTE_INVALID_PARAMETER_NAME = "Invalid parameter name (See RFC 2426 section 4 for param-name syntax)"
NOTE_MISMATCH_GROUP = "Group mismatch (See RFC 2426 section 4 for contentline syntax)"

# Values & Sub-values
NOTE_INVALID_PARAMETER_VALUE = "Invalid parameter value (See RFC 2426 section 4 for param-value syntax)"
NOTE_MISMATCH_PARAMETER = "Parameter mismatch (See RFC 2426 section 3 for details)"
NOTE_INVALID_DATE = "Invalid date (See RFC 2425 section 5.8.4 for date syntax)"
NOTE_INVALID_LANGUAGE_VALUE = "Invalid language (See RFC 1766 section 2 for details)"
NOTE_INVALID_SUB_VALUE = "Invalid sub-value (See RFC 2426 section 3 for details)"
NOTE_INVALID_TEXT_VALUE = "Invalid text value (See RFC 2426 section 4 for details)"
NOTE_INVALID_TIME = "Invalid time (See RFC 2425 section 5.8.4 for time syntax)"
NOTE_INVALID_TIME_ZONE = "Invalid time zone (See RFC 2426 section 3.4.1 for time-zone syntax)"
NOTE_INVALID_URI = "Invalid URI (See RFC 1738 section 5 for genericurl syntax)"
NOTE_INVALID_VALUE = "Invalid value (See RFC 2426 section 3 for details)"

# Warning literals
WARN_DEFAULT_TYPE_VALUE = "Using default TYPE value; can be removed"
WARN_INVALID_DATE = "Possible invalid date"
WARN_INVALID_EMAIL_TYPE = "Possible invalid email TYPE"
WARN_MULTIPLE_NAMES = "Possible split name (replace space with comma)"


def _stringify(text):
    """
    Get the text as a string representation

    @param text: Something convertible to str
    @return: Printable string
    """
    try:
        text = str(text)
    except UnicodeEncodeError:
        text = text.encode("utf-8")
    return text


def show_warning(message, category=UserWarning, filename="", lineno=-1, file=sys.stderr, line=None):
    """Custom simple warning."""
    file.write("{0}\n".format(message))


class VCardError(Exception):
    """Raised if the text given is not a valid according to RFC 2426."""

    def __init__(self, message, context):
        """
        vCard format error.

        @param message: Error message
        @param context: Dictionary with context information
        """
        Exception.__init__(self)
        self.message = message
        self.context = context

    def __str__(self):
        """
        Outputs error with ordered context info.

        @return: Printable error message
        """
        message = _stringify(self.message)

        # Sort context information
        keys = ["File", "File line", "vCard line", "Property", "Property line", "String"]
        for key in keys:
            if key in self.context:
                message += "\n{0}: {1}".format(_stringify(key), _stringify(self.context.pop(key)))

        return message


class VCardLineError(VCardError):
    """Raised for line-related errors."""

    pass


class VCardNameError(VCardError):
    """Raised for name-related errors."""

    pass


class VCardValueError(VCardError):
    """Raised for value-related errors."""

    pass


class VCardItemCountError(VCardError):
    """Raised when a required number of something is not present."""

    pass


class UsageError(Exception):
    """Raise in case of invalid parameters."""

    def __init__(self, message):
        Exception.__init__(self)
        self._message = message

    def __str__(self):
        return self._message



import codecs
import re
import sys
import warnings

from . import vcard_utils, vcard_validators
from .vcard_definitions import (
    ALL_PROPERTIES,
    ID_CHARACTERS,
    MANDATORY_PROPERTIES,
    NEWLINE_CHARACTERS,
    QUOTE_SAFE_CHARACTERS,
    SAFE_CHARACTERS,
    SPACE_CHARACTER,
    VALUE_CHARACTERS,
    VCARD_LINE_MAX_LENGTH_RAW,
)
from .vcard_errors import (
    NOTE_CONTINUATION_AT_START,
    NOTE_DOT_AT_LINE_START,
    NOTE_EMPTY_VCARD,
    NOTE_INVALID_LINE_SEPARATOR,
    NOTE_INVALID_PARAMETER_NAME,
    NOTE_INVALID_PROPERTY_NAME,
    NOTE_INVALID_SUB_VALUE,
    NOTE_INVALID_VALUE,
    NOTE_MISMATCH_GROUP,
    NOTE_MISSING_GROUP,
    NOTE_MISSING_PARAM_VALUE,
    NOTE_MISSING_PROPERTY,
    NOTE_MISSING_VALUE_STRING,
    VCardError,
    VCardItemCountError,
    VCardLineError,
    VCardNameError,
    VCardValueError,
)
from .vcard_property import VcardProperty


class VcardValidator(object):
    def __init__(self, path, verbose):
        self.path = path
        self.verbose = verbose
        self.result = self.validate()

    def validate(self):
        return validate_file(self.path, self.verbose)


def validate_file(filename, verbose):
    """
    Create object for each vCard in a file, and show the error output.

    @param filename: Path to file
    @param verbose: Verbose mode
    @return: Debugging output from creating vCards
    """
    if filename == "-":
        file_pointer = sys.stdin
    else:
        file_pointer = codecs.open(filename, "r", "utf-8")

    contents = file_pointer.read().splitlines(True)

    vcard_text = ""
    result = ""
    try:
        for index in range(len(contents)):
            line = contents[index]
            vcard_text += line

            if line == NEWLINE_CHARACTERS:
                try:
                    vcard = VCard(vcard_text, filename)
                    vcard_text = ""
                    if verbose:
                        print(vcard)
                except VCardError as error:
                    error.context["File"] = filename
                    error.context["File line"] = index
                    err_type = type(error)
                    raise err_type(error.message, error.context)

    except VCardError as error:
        result += str(error)

    if vcard_text != "" and result == "":
        result += "Could not process entire %s - %i lines remain" % (filename, len(vcard_text.splitlines(False)))

    if result == "":
        return None
    return result


class VCard:
    """Container for structured and unstructured vCard contents."""

    def __init__(self, text, filename=None):
        """
        Create vCard object from text string. Includes text (the entire
        unprocessed vCard), group (optional prefix on each line) and
        properties.

        @param text: String containing a single vCard
        """
        if text == "" or text is None:
            raise VCardError(NOTE_EMPTY_VCARD, {"vCard line": 1, "File line": 1})

        self.text = text

        self.filename = filename

        lines = unfold_vcard_lines(self.text.splitlines(True))

        # Groups
        self.group = get_vcard_group(lines)
        lines = remove_vcard_groups(lines, self.group)

        # Properties
        self.properties = get_vcard_properties(lines)

    def __str__(self):
        return self.text.encode("utf-8")


def unfold_vcard_lines(lines):
    """
    Un-split lines in vCard, warning about short lines. RFC 2426 page 8.

    @param lines: List of potentially folded vCard lines
    @return: List of lines, one per property
    """
    property_lines = []
    for index in range(len(lines)):
        line = lines[index]
        if not line.endswith(NEWLINE_CHARACTERS):
            raise VCardLineError(NOTE_INVALID_LINE_SEPARATOR, {"File line": index + 1})

        if len(line) > VCARD_LINE_MAX_LENGTH_RAW:
            warnings.warn("Long line in vCard: {0}".format(line.encode("utf-8")))

        if line.startswith(" "):
            if index == 0:
                raise VCardLineError(NOTE_CONTINUATION_AT_START, {"File line": index + 1})
            elif len(lines[index - 1]) < VCARD_LINE_MAX_LENGTH_RAW:
                warnings.warn("Short folded line at line {0:d}".format(index - 1))
            elif line == SPACE_CHARACTER + NEWLINE_CHARACTERS:
                warnings.warn("Empty folded line at line {0:d}".format(index))
            property_lines[-1] = property_lines[-1][: -len(NEWLINE_CHARACTERS)] + line[1:]
        else:
            property_lines.append(line)

    return property_lines


def get_vcard_group(lines):
    """
    Get & validate group. RFC 2426 pages 28, 29.

    @param lines: List of unfolded vCard lines
    @return: Group name if one exists, None otherwise
    """
    group = None

    group_re = re.compile("^([{0}]*)\\.".format(re.escape(ID_CHARACTERS)))

    group_match = group_re.match(lines[0])
    if group_match is not None:
        group = group_match.group(1)

        # Validate
        if len(group) == 0:
            raise VCardLineError(NOTE_DOT_AT_LINE_START, {})

        for index in range(len(lines)):
            line = lines[index]
            next_match = group_re.match(line)
            if not next_match:
                raise VCardLineError(NOTE_MISSING_GROUP, {"File line": index + 1})
            if next_match.group(1) != group:
                raise VCardNameError(
                    "{0}: {1} != {2}".format(NOTE_MISMATCH_GROUP, next_match.group(1), group), {"File line": index + 1}
                )
    else:
        # Make sure there are no groups elsewhere
        for index in range(len(lines)):
            next_match = group_re.match(lines[index])
            if next_match:
                raise VCardNameError(
                    "{0}: {1} != {2}".format(NOTE_MISMATCH_GROUP, next_match.group(1), group), {"File line": index + 1}
                )

    return group


def remove_vcard_groups(lines, group):
    """
    Remove groups from vCard. RFC 2426 pages 28, 29.

    @param lines: List of unfolded vCard lines
    @param group: Name of group
    @return: Lines without groups and dot separator
    """
    if group:
        for index in range(len(lines)):
            lines[index] = lines[index][len(group) :]
    return lines


def get_vcard_properties(lines):
    """
    Get the properties for each line. RFC 2426 pages 28, 29.

    @param lines: List of unfolded vCard lines
    @return: List of properties, for simplicity. AFAIK sequence doesn't matter
    and duplicates add no information, but ignoring this to make sure vCard
    output looks like the original.
    """
    properties = []
    for index in range(len(lines)):
        property_line = lines[index]
        if property_line != NEWLINE_CHARACTERS:
            try:
                properties.append(get_vcard_property(property_line))
            except VCardError as error:
                error.context["vCard line"] = index
                err_type = type(error)
                raise err_type(error.message, error.context)

    for mandatory_property in MANDATORY_PROPERTIES:
        if mandatory_property not in [property_.name.upper() for property_ in properties]:
            raise VCardItemCountError(
                "{0}: {1}".format(NOTE_MISSING_PROPERTY, mandatory_property), {"Property": mandatory_property}
            )

    return properties


def get_vcard_property(property_line):
    """
    Get a single property.

    @param property_line: Single unfolded vCard line
    @return: Dictionary with name, parameters and values
    """
    property_parts = vcard_utils.split_unescaped(property_line, ":")
    if len(property_parts) < 2:
        raise VCardItemCountError("{0}: {1}".format(NOTE_MISSING_VALUE_STRING, property_line), {})
    elif len(property_parts) > 2:
        # Merge - Colon doesn't have to be escaped in values
        property_parts[1] = ":".join(property_parts[1:])
        property_parts = property_parts[:2]
    property_string, values_string = property_parts

    # Split property name and property parameters
    property_name_and_params = vcard_utils.split_unescaped(property_string, ";")

    property_ = VcardProperty(property_name_and_params.pop(0))

    # String validation
    if not property_.name.upper() in ALL_PROPERTIES and not re.match(
        "^X-[{0}]+$".format(re.escape(ID_CHARACTERS)), property_.name, re.IGNORECASE
    ):
        raise VCardNameError("{0}: {1}".format(NOTE_INVALID_PROPERTY_NAME, property_.name), {})

    try:
        if len(property_name_and_params) != 0:
            property_.parameters = get_vcard_property_params(";".join(property_name_and_params))
        property_.values = get_vcard_property_values(values_string)

        # Validate
        vcard_validators.validate_vcard_property(property_)
    except VCardError as error:
        # Add parameter name to error
        error.context["Property line"] = property_line
        err_type = type(error)
        raise err_type(error.message, error.context)

    return property_


def get_vcard_property_params(params_string):
    """
    Get the parameters and their values. RFC 2426 page 28.

    @param params_string: Part of a vCard line between the first semicolon
    and the first colon.
    @return: Dictionary of parameters. Assumes that
    TYPE=WORK;TYPE=WORK,VOICE === TYPE=VOICE,WORK === TYPE=VOICE;TYPE=WORK.
    """
    params = {}
    if not params_string:
        return params

    for parameter_string in vcard_utils.split_unescaped(params_string, ";"):
        parameter = get_vcard_property_parameter(parameter_string)
        param_name = parameter["name"].upper()  # To be able to merge TYPE & type
        if param_name not in params:
            params[param_name] = parameter["values"]
        else:
            # Merge
            params[param_name] = params[param_name].union(parameter["values"])

    return params


def get_vcard_property_values(values_string):
    """
    Get the property values.

    @param values_string: Multiple value string
    @return: List of values (RFC 2426 page 12)
    """
    values = []

    # Strip line ending
    values_string = values_string[: -len(NEWLINE_CHARACTERS)]

    sub_value_strings = vcard_utils.split_unescaped(values_string, ";")
    for sub in sub_value_strings:
        values.append(get_vcard_property_sub_values(sub))

    return values


def get_vcard_property_parameter(param_string):
    """
    Get the parameter name and value(s). RFC 2426 page 29.

    @param param_string: Single parameter and values
    @return: Dictionary with a parameter name and values
    """
    try:
        param_name, values_string = vcard_utils.split_unescaped(param_string, "=")
    except ValueError as error:
        raise VCardItemCountError("{0}: {1}".format(NOTE_MISSING_PARAM_VALUE, str(error)), {})

    values = get_vcard_property_param_values(values_string)

    # Validate
    if not re.match("^[{0}]+$".format(re.escape(ID_CHARACTERS)), param_name):
        raise VCardNameError("{0}: {1}".format(NOTE_INVALID_PARAMETER_NAME, param_name), {})

    return {"name": param_name, "values": values}


def get_vcard_property_sub_values(value_string):
    """
    Get the parts of the value.

    @param value_string: Single value string
    @return: List of values (RFC 2426 page 9)
    """
    sub_values = vcard_utils.split_unescaped(value_string, ",")

    # Validate string
    for sub_value in sub_values:
        if not re.match(u"^[{0}]*$".format(re.escape(VALUE_CHARACTERS)), sub_value):
            raise VCardValueError("{0}: {1}".format(NOTE_INVALID_SUB_VALUE, sub_value), {})

    return sub_values


def get_vcard_property_param_values(values_string):
    """
    Get the parameter values. RFC 2426 page 29.

    @param values_string: Comma separated values
    @return: Set of values. Assumes that sequence doesn't matter and that
    duplicate values can be discarded, even though RFC 2426 doesn't explicitly
    say this. I.e., assumes that TYPE=WORK,VOICE,WORK === TYPE=VOICE,WORK.
    """
    values = set(vcard_utils.split_unescaped(values_string, ","))

    # Validate
    for value in values:
        if not re.match(
            u'^[{0}]+$|^"[{1}]+"$'.format(re.escape(SAFE_CHARACTERS), re.escape(QUOTE_SAFE_CHARACTERS)), value
        ):
            raise VCardValueError("{0}: {1}".format(NOTE_INVALID_VALUE, value), {})

    return values



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""vCard v3.0 (RFC 2426) definitions and message strings"""


def character_range(start, end):
    return "".join(chr(index) for index in range(start, end + 1))


# Literals, RFC 2426 pages 27, 28
ALPHA_CHARACTERS = character_range(0x41, 0x5A) + character_range(0x61, 0x7A)
CARRIAGE_RETURN_CHARACTER = chr(0x0D)
LINE_FEED_CHARACTER = chr(0x0A)
NEWLINE_CHARACTERS = CARRIAGE_RETURN_CHARACTER + LINE_FEED_CHARACTER
CONTROL_CHARACTERS = character_range(0x00, 0x1F) + chr(0x7F)
DIGIT_CHARACTERS = character_range(0x30, 0x39)
DOUBLE_QUOTE_CHARACTER = chr(0x22)
HORIZONTAL_TAB_CHARACTER = chr(0x09)
SPACE_CHARACTER = chr(0x20)
PRINTABLE_CHARACTERS = character_range(0x21, 0x7E)
WHITESPACE_CHARACTERS = SPACE_CHARACTER + HORIZONTAL_TAB_CHARACTER
NON_ASCII_CHARACTERS = character_range(0x80, 0xFF)

# Have to remove backslash!
QUOTE_SAFE_CHARACTERS = (
    WHITESPACE_CHARACTERS + chr(0x21) + character_range(0x23, 0x5B) + character_range(0x5D, 0x7E) + NON_ASCII_CHARACTERS
)
SAFE_CHARACTERS = (
    WHITESPACE_CHARACTERS
    + chr(0x21)
    + character_range(0x23, 0x2B)
    + character_range(0x2D, 0x39)
    + character_range(0x3C, 0x5B)
    + character_range(0x5D, 0x7E)
    + NON_ASCII_CHARACTERS
)
VALUE_CHARACTERS = WHITESPACE_CHARACTERS + PRINTABLE_CHARACTERS + NON_ASCII_CHARACTERS
ESCAPED_CHARACTERS = u"\\;,nN"

# Known property names (RFC 2426 page 4)
MANDATORY_PROPERTIES = ["BEGIN", "END", "FN", "N", "VERSION"]
PREDEFINED_PROPERTIES = ["BEGIN", "END", "NAME", "PROFILE", "SOURCE"]
OTHER_PROPERTIES = [
    "ADR",
    "AGENT",
    "BDAY",
    "CATEGORIES",
    "CLASS",
    "EMAIL",
    "GEO",
    "KEY",
    "LABEL",
    "LOGO",
    "MAILER",
    "NICKNAME",
    "NOTE",
    "ORG",
    "PHOTO",
    "PRODID",
    "REV",
    "ROLE",
    "SORT-STRING",
    "SOUND",
    "TEL",
    "TITLE",
    "TZ",
    "UID",
    "URL",
]
ALL_PROPERTIES = list(set(MANDATORY_PROPERTIES + PREDEFINED_PROPERTIES + OTHER_PROPERTIES))

# IDs for group, name, iana-token, x-name, param-name (RFC 2426 page 29)
ID_CHARACTERS = ALPHA_CHARACTERS + DIGIT_CHARACTERS + "-"

VCARD_LINE_MAX_LENGTH = 75
"""RFC 2426 page 6"""

VCARD_LINE_MAX_LENGTH_RAW = VCARD_LINE_MAX_LENGTH + len(NEWLINE_CHARACTERS)
"""Including line ending"""



#!/usr/bin/env python
import argparse
import sys

from .vcard_errors import UsageError
from .vcard_validator import VcardValidator

PATH_ARGUMENT_HELP = "The files to validate. Use '-' for standard input"
VERBOSE_OPTION_HELP = "Enable verbose output"


def main():
    try:
        arguments = parse_arguments(sys.argv[1:])
    except UsageError as error:
        sys.stderr.write("{0}\n".format(str(error)))
        return 2

    return_code = 0
    for filename in arguments.paths:
        result = VcardValidator(filename, arguments.verbose).result
        if result is not None:
            print(result)
            return_code = 1

    return return_code


def parse_arguments(arguments):
    argument_parser = argparse.ArgumentParser()
    argument_parser.add_argument("--verbose", default=False, action="store_true", help=VERBOSE_OPTION_HELP)
    argument_parser.add_argument("paths", metavar="path", nargs="+", help=PATH_ARGUMENT_HELP)
    try:
        parsed_arguments = argument_parser.parse_args(args=arguments)
    except argparse.ArgumentError as error:
        raise UsageError(str(error))
    return parsed_arguments


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
"""Setup configuration"""

from os import path

from setuptools import setup

from vcard import __author__, __email__, __license__, __maintainer__, __package__, __url__, __version__

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as file_handle:
    long_description = file_handle.read()

setup(
    name=__package__,
    version=__version__,
    description="vCard validator, class and utility functions",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=__url__,
    keywords="vCard vCards RFC 2426 RFC2426 validator",
    packages=[__package__],
    setup_requires=[],
    install_requires=["python-dateutil"],
    entry_points={"console_scripts": ["%(package)s=%(package)s.%(package)s:main" % {"package": __package__}]},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Topic :: Text Processing",
        "Topic :: Utilities",
    ],
    test_suite="tests",
    author=__author__,
    author_email=__email__,
    maintainer=__maintainer__,
    maintainer_email=__email__,
    download_url="http://pypi.python.org/pypi/vcard/",
    platforms=["POSIX", "Windows"],
    license=__license__,
    obsoletes=["vcard_module"],
)
